/*****************************************************************************
*
* Copyright (C) 2003 Atmel Corporation
*
* File              : TerminalDriver.c
* Compiler          : IAR EWAAVR 2.28a
* Version           : 2.1          
* Last modified     : 18-07-2003 by LTA
*
* Support mail      : avr@atmel.com
*
* Supported devices : ATmega128 (Modify UART register defines to add support for other devices)
*
* Description       : Terminal emulation interface
*                     
*
*
****************************************************************************/

#include "TerminalDriver.h"


//***************************
// Misc definitions
// Change for other devices
//***************************

/*** UART registers ***/
#define UARTBAUD        UBRRL           // UART Baud Rate Register Low
#define UARTSTATUS      UCSRA           // UART Status Register
#define UARTCONTROL     UCSRB           // UART Control Register
#define UARTDATA        UDR             // UART Data Register

#define UARTTXEN        TXEN            // Transmitter Enable Bit
#define UARTRXEN        RXEN            // Receiver Enable Bit
#define UARTTXC         TXC             // Transmit Complete Bit
#define UARTRXC         RXC             // Receive Complete Bit
#define UARTUDRE        UDRE            // UART Data Register Empty Bit

//#define BAUDRATEFACTOR    8              // Calculated baud rate factor: 115200bps @ 16 MHz
#define BAUDRATEFACTOR    51             // Calculated baud rate factor: 19200bps @ 16 MHz
//#define BAUDRATEFACTOR    103             // Calculated baud rate factor: 9600bps @ 16 MHz



//***************************
// Macros
//***************************

/*** Send escape sequence start ***/
#define SENDESC            \
    Term_Send( 27 );       \
    Term_Send( '[' );


//***************************
// Initialize UART Terminal
//***************************
void Term_Initialise(void)
{
    UARTBAUD = BAUDRATEFACTOR;                    // Set baud rate and enable UART
    UARTCONTROL = (1<<UARTTXEN) | (1<<UARTRXEN);
    
    //Term_Set_Display_Attribute_Mode( MODE_NONE ); // Disable all previous modes
    Term_Erase_Screen();                          // Clear screen
    //Term_Set_Cursor_Position( 1, 1 );             // Move to top-left corner
}


//****************************
// Transmit one byte
//****************************
void Term_Send( unsigned char data )
{
    while( !(UARTSTATUS & (1<<UARTUDRE)) );       // Wait for data register ready
        
    UARTDATA = data;                              // Send byte
}


//***************************
// Receive one byte
//***************************
unsigned char Term_Get(void)
{
    while( !(UARTSTATUS & (1<<UARTRXC)) );        // Wait for receive complete
        
    return UARTDATA;                              // Return byte
}

//***************************
// Transmit string from SRAM
//
// Returns byte count
//***************************
unsigned char Term_Send_RAMStr( unsigned char * str )
{
    unsigned char count = 0;            // Initialize byte counter

    while( *str != 0 )                  // Reached zero byte ?
    {
        Term_Send( *str );
        str++;                          // Increment ptr
        count++;                        // Increment byte counter
    }    
    
    return count;                       // Return byte count
}



//***************************
// Send 'clear screen' sequence
//***************************
void Term_Erase_Screen(void)
{
//    SENDESC                             // Send escape sequence start
    
//    Term_Send( '2' );
//    Term_Send( 'J' );

	unsigned char i;	

	for(i=0; i<15; i++) {
		//Term_Send_RAMStr(BlankLine);
		strcpy_P(tempstr, (PGM_P)BlankLine);
		Term_Send_RAMStr(tempstr);
	}
}


