/*****************************************************************************
*
* Copyright (C) 2010 Chris Troutner
*
* Version 0.03 moves the 1ms timing and ADC switching to Timer/Counter 0.
* Timer/Counter1 is used to create a 25KHz PWM signal on PB1/OC1A to drive
* the MOSFET.
*                     
*
*
****************************************************************************/

#include <string.h>
//#include <stdlib.h>
//#include <stdio.h>
//#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h> 

//#define F_CPU 16000000UL
//#include <util/delay.h>

//#include "delays.h"
//#include "delays.c"

void Init_ADC(void);
unsigned char TimeCheck(unsigned long starttime, unsigned int elapsedtime);
void DisplaySensors(void);
unsigned char Term_Get_With_Time(unsigned long starttime, unsigned int elapsedtime);
void LPF(float sensor[]);
float ConvertVoltage(int ADCData);
void UpdateSensorData(unsigned char sensor);


//Define String Constants
const char menu[] PROGMEM = "1) Read Sensors\n2) Choice B\n3) Choice C\n";
const char return_instruct[] PROGMEM = "\n\nPress Enter to return to the main menu.";
const char BlankLine[] PROGMEM = "                                                  \n";
const char PanelStr[] PROGMEM = "Panel ";
const char BattStr[] PROGMEM = "Battery ";
const char VoltageStr[] PROGMEM = "Voltage: ";
const char CurrentStr[] PROGMEM = "Current: ";
const char UnitVolts[] PROGMEM = " Volts\n";
const char UnitAmps[] PROGMEM = " Amps\n";



unsigned char tempstr[80] = "\n";
float tempdata;
unsigned char i = 0;
unsigned long mscount = 0;

//SensorData
//Follows the following data structure: [y(n) y(n-1) y(n-2) x(n) x(n-1) x(n-2)]
float ADCChan[4][6] = {{0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0}};


#include "TerminalDriver.c"

//Timer/Counter1 Compare Match Interrupt Handler
//Generated every 1 mS to signal service of functions in main()
ISR(SIG_OVERFLOW0)
{	
	static unsigned char ISRCnt = 0;

	mscount++;

	ISRCnt++;

	switch (ISRCnt) {
	
		case 25:
			UpdateSensorData(0);
			
			//Switch ADC to channel 1
			ADMUX |= 1<<MUX0;

			break;

		case 50:
			UpdateSensorData(1);

			//Switch ADC to channel 2
			ADMUX &= ~(1<<MUX0);
			ADMUX |= (1<<MUX1);
			break;

		case 75:
			UpdateSensorData(2);			

			//Switch ADC to channel 3
			ADMUX |= (1<<MUX0);
			ADMUX |= (1<<MUX1);
			break;

		case 100:
			UpdateSensorData(3);

			//Switch ADC to channel 0
			ADMUX &= ~(1<<MUX0);
			ADMUX &= ~(1<<MUX1);

			ISRCnt = 0;
			break;

	}

	
	/*	
	itoa(mscount, tempstr, 10);	
	strcat(tempstr, "\n");				
	Term_Send_RAMStr(tempstr);
	*/

	//Clear Timer Counter 1	
	TCNT0 = 240;

}

/*
This interrupt handler will be used to dynamically adjust drive frequency
and duty cycle.
*/
ISR(TIMER1_COMPA_vect) {

	//static unsigned char blah = 0;

	//blah++;

}



int main(void)
{
    unsigned char ret;
	unsigned long temptime;


	//Initialize Timer/Counter1 for 1ms interrupt
	TCCR0 = (1<<CS02)|(1<<CS00); //Prescale by 1024. 16Mhz --> 16Khz
	TCNT0 = 240; //Set Counter so that it overflows after 16 counts.
	TIMSK |= (1<<TOIE0);	//Enable overflow interrupt.

	DDRB |= 0x02; //Set PB1/OC1A as an output. Drives a low by default.
	
	//Initialize Timer/Counter1 to generate a 25KHz drive signal
	TCCR1A |= (1<<COM1A0); //Set to toggle 0C1A on Compare Match.
	TCCR1B |= (1<<CS10); //Set clocking from main osc. No prescaling.
	TCCR1B |= (1<<WGM12); //Set CTC mode using the OCR1A register.
	OCR1A = 320; //Set Compare Match for 320 == 25KHz
	TIMSK |= (1<<OCIE1A); //Enable interrupt on compare match
	
	
	//Initialize the ADC	
	Init_ADC();	

	//Enable interrupts
	SREG |= (1<<7);

	Term_Initialise();                            // Initialize terminal server

    while(1) {    


		/* Handle menu and print result */
		//ret = Term_Handle_Menu( menu, 1, 10, 10, 1 );
		strcpy_P(tempstr, (PGM_P)menu);
		Term_Send_RAMStr(tempstr);
		ret = Term_Get();
		Term_Send(ret);

		Term_Erase_Screen();

		//If the user selects to display sensor data:
		if (ret == '1') {

			//Record the current elapsed time in ms.
			temptime = mscount;			

			do {

				//Display sensor data.
				DisplaySensors();


				//Wait until user hits Esc button
				//Term_Send_RAMStr("\n\n");		
				strcpy_P(tempstr, (PGM_P)return_instruct);
				Term_Send_RAMStr(tempstr);

				ret = Term_Get_With_Time(temptime, 2000);
				//ret = Term_Get();

				temptime = mscount;
	
			} while (ret != 13);

		}

	
		Term_Erase_Screen();

	}
       


        //Term_Send( Term_Get() );                // Echo forever...

}



void Init_ADC(void) {

	//ADMUX = 0;
	ADMUX |= 1<<REFS0;
	
	ADCSRA |= 1<<ADEN; //Enable ADC
	ADCSRA |= 1<<ADSC; //Start Conversion
	ADCSRA |= 1<<ADFR; //Select ADC Free Running Mode
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); //Prescale by 128 to get 125KHz rate	
	


	//itoa(ADCSRA, tempstr, 10);	
	//Term_Send_RAMStr(tempstr);


}

/*
TimeCheck returns 0 or 1 depending on weather or not the elapsed time has passed.
starttime should be a copy of the mscount variable at the start of calling.
elapsedtime is the time in ms that you want to wait.
*/
unsigned char TimeCheck(unsigned long starttime, unsigned int elapsedtime) {

	unsigned long temptime;

	temptime = mscount - starttime;

	if (temptime >= elapsedtime)
		return 1;
	else
		return 0;

}

//This function displays sensor data
void DisplaySensors(void) {

		int CurADC;
		//float Voltage;
		float Current;
		char curstr[10];

		//Initialize the screen		
		Term_Erase_Screen();
		Term_Send_RAMStr("\n");		

		//Read ADC		
		//CurADC = ADC;
		
		//Switch ADC to channel 1
		//ADMUX |= 1<<MUX0;

		//Display Battery Voltage
		strcpy_P(tempstr, (PGM_P)PanelStr);
		strcat_P(tempstr, (PGM_P)VoltageStr);
		//Voltage = CurADC*0.04883;
		dtostrf(ADCChan[1][0], 5, 2, curstr);
		strcat(tempstr, curstr);
		strcat_P(tempstr, (PGM_P)UnitVolts);
		Term_Send_RAMStr(tempstr);

		//Read ADC
		//CurADC = ADC;
		
		//Switch ADC to channel 2
		//ADMUX &= ~(1<<MUX0);
		//ADMUX |= (1<<MUX1);

		//Display Battery Current
		strcpy_P(tempstr, (PGM_P)PanelStr);
		strcat_P(tempstr, (PGM_P)CurrentStr);
		//Current = CurADC*0.04883 - Voltage;
		Current = ((ADCChan[1][0]-ADCChan[0][0])/2.0);
		dtostrf(Current, 5, 2, curstr);
		//dtostrf(ADCChan[0][0],5,2,curstr);
		strcat(tempstr, curstr);
		strcat_P(tempstr, (PGM_P)UnitAmps);
		Term_Send_RAMStr(tempstr);

		//Read ADC
		//CurADC = ADC;
		
		//Switch ADC to channel 3
		//ADMUX |= (1<<MUX0);
		//ADMUX |= (1<<MUX1);

		strcpy_P(tempstr, (PGM_P)BattStr);
		strcat_P(tempstr, (PGM_P)VoltageStr);
		//Voltage = CurADC*0.04883;
		dtostrf(ADCChan[3][0]/2.0, 5, 2, curstr);
		strcat(tempstr, curstr);
		strcat_P(tempstr, (PGM_P)UnitVolts);
		Term_Send_RAMStr(tempstr);


		//Read ADC
		//CurADC = ADC;
		
		//Switch ADC to channel 0
		//ADMUX &= ~(1<<MUX0);
		//ADMUX &= ~(1<<MUX1);

		strcpy_P(tempstr, (PGM_P)BattStr);
		strcat_P(tempstr, (PGM_P)CurrentStr);
		Current = (2.0*(ADCChan[2][0]-ADCChan[3][0]));
		dtostrf(Current, 5, 2, curstr);
		//dtostrf(ADCChan[2][0],5,2,curstr);
		strcat(tempstr, curstr);
		strcat_P(tempstr, (PGM_P)UnitAmps);
		Term_Send_RAMStr(tempstr);

}


/*
This function waits to get a character from a user. However, it will 
also time out and return 0 if the user does not press anything.
*/
unsigned char Term_Get_With_Time(unsigned long starttime, unsigned int elapsedtime)
{
    while( !(UARTSTATUS & (1<<UARTRXC)) && !TimeCheck(starttime, elapsedtime) );        // Wait for receive complete
        
	if( TimeCheck(starttime, elapsedtime) )
		return 0;
	else
    	return UARTDATA;                              // Return byte
}


void LPF(float sensor[]) {

	//For a fc or 1/3*fs	
	//const float beta = 0.097908;
	//const float gamma = -0.184764;
	//const float alpha = 0.195668;

	
	//For fc/fs = 2/10
	const float beta = 0.097908;
	const float gamma = 0.184764;
	const float alpha = 0.103286;
	

	/*
	//For fc/fs = 1/10 - causes overflow
	const float beta = 0.206401;
	const float gamma = 0.57149;
	const float alpha = 0.033728;
	*/

	/*
	//For fc/fs = 1/100 - doesn't work.
	const float beta = 0.457488;
	const float gamma = 0.955599;
	const float alpha = 0.000472;
	*/

	float temp;	
	
	temp = sensor[3]+2*sensor[4]+sensor[5];
	temp *= alpha;
	temp += gamma*sensor[1];
	temp -= beta*sensor[2];
	temp *= 2;

	sensor[2] = sensor[1];
	sensor[1] = sensor[0];
	sensor[0] = temp;
}

float ConvertVoltage(int ADCData) {


	//return ADCData*0.04883;
	return ADCData*0.04882813;

}

void UpdateSensorData(unsigned char sensor) {

			ADCChan[sensor][5] = ADCChan[sensor][4];
			ADCChan[sensor][4] = ADCChan[sensor][3];
			ADCChan[sensor][3] = ConvertVoltage(ADC);
			LPF(&ADCChan[sensor][0]);

}

